# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Lukas Brabec <lbrabec@redhat.com>

import json
import os
import requests

from flask import request, jsonify, make_response

from oraculum import app, oidc


def get_kanban_config():
    kanban_config_raw = os.getenv('KANBAN_CONFIG')
    if not kanban_config_raw:
        app.logger.error('KANBAN_CONFIG variable not provided, sending empty dict')
        return {}

    return json.loads(kanban_config_raw)


@app.route('/api/v1/kanban')
def api_v1_kanban():
    kanban_config_out = get_kanban_config()
    for project in kanban_config_out.values():
        # don't send token to frontend
        project.pop('token')

        resp = requests.get(project['pagure_api'] + 'issues')
        project['issues'] = []
        if resp.status_code != 200:
            app.logger.error('Unable to get issues of %s' % project['url'])
        else:
            project['issues'] = resp.json()['issues']
            for issue in project['issues']:
                # for frontend convenience
                issue['stage'] = project['default_stage']
                for cf in issue['custom_fields']:
                    if cf['name'] == 'stage':
                        issue['stage'] = cf['value']

    return jsonify(kanban_config_out)


@app.route('/api/v1/kanban/move_issue', methods=['OPTIONS'])
def api_v1_kanban_move_issue_options():
    response = make_response()

    response.headers.add('Access-Control-Allow-Headers', 'Content-Type,Authorization')
    response.headers.add('Access-Control-Allow-Methods', 'POST,OPTIONS')

    return response


@app.route('/api/v1/kanban/move_issue', methods=['POST'])
@oidc.accept_token(require_token=True)
def api_v1_kanban_move_issue():
    data = request.json
    project = data['project']
    issue_id = data['issue_id']
    new_stage = data['new_stage']
    # FIXME use better data structures (or better config json)
    kanban_config = get_kanban_config()
    token = kanban_config[project]['token']
    app.logger.debug('Project %s: move issue %s to stage %s, using token %s' % (project, issue_id, new_stage, token))

    update_stage_url = kanban_config[project]['pagure_api'] + 'issue/%s/custom/stage' % issue_id
    pagure_resp = requests.post(update_stage_url,
                                headers={
                                    'Authorization': 'token %s' % token
                                },
                                data={'value': new_stage})

    resp = data
    if 'err' in pagure_resp.json():
        resp['new_stage'] == resp['old_stage']
        app.logger.error('Unable to update issue, response code: %d, '
                         'response text:\n%s' % (pagure_resp.status_code, pagure_resp.text))
    return jsonify(resp)
