# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Josef Skladanka <jskladan@redhat.com>

import urllib

from flask import url_for, jsonify

from oraculum import app, CACHE
from oraculum.utils import dashboard_helpers, pagure, koschei, health_check, bodhi, bugzilla

from oraculum.action_providers import ACTION_PROVIDERS

from oraculum.controllers.landing_page import api_v1_landing_page
from oraculum.controllers.libkarma import all_bodhi_updates
from oraculum.controllers.packager_dashboard import dashboard_user_data_static, handle_orphan_user


@app.before_first_request
def app_before_first_request():
    app.logger.debug('Registering cache refreshers')
    register_cache_providers()


def register_cache_providers():
    """
    Add providers to be cached here
    """
    CACHE.register('fedora_releases', dashboard_helpers.get_fedora_releases)
    CACHE.register('landing_page', api_v1_landing_page)
    for p_name, p_module in ACTION_PROVIDERS.items():
        CACHE.register(p_name, p_module.get_actions)

    CACHE.register('pagure_groups', pagure.get_pagure_groups)
    CACHE.register('packages_owners_json',
                   lambda url=app.config['PACKAGE_MAINTAINERS_JSON_URL']: dashboard_helpers.get_json(url))
    CACHE.register('orphans_json', lambda url=app.config['ORPHANS_JSON_URL']: dashboard_helpers.get_json(url))
    CACHE.register('koschei_data', koschei.parse_koschei_data)
    CACHE.register('health_check_data', health_check.get_health_check_data)
    CACHE.register('bodhi_overrides', bodhi.get_overrides)
    CACHE.register('bodhi_updates', all_bodhi_updates)
    CACHE.register('ftbfs_trackers', bugzilla.get_ftbfs_trackers)
    CACHE.register('fti_trackers', bugzilla.get_fti_trackers)
    CACHE.register('blocker_trackers', bugzilla.get_blocker_trackers)

    CACHE.register('packager-dashboard_user_data_static', dashboard_user_data_static)
    CACHE.register('packager-dashboard_bugs', bugzilla.get_package_bugs)
    CACHE.register('packager-dashboard_pull_requests', pagure.get_package_prs)
    CACHE.register('packager-dashboard_package_versions', pagure.get_package_versions)
    CACHE.register('packager-dashboard_orphan_page', handle_orphan_user)


@app.route('/api/v1/releases')
def route_api_v1_releases():
    return jsonify({"fedora": CACHE.get('fedora_releases'), "epel": app.config["EPEL_RELEASES"]})

@app.route('/')
@app.route('/index')
def index():
    endpoints = []

    for rule in app.url_map.iter_rules():

        if rule.endpoint in ['index', 'static']:
            continue

        options = {}
        for arg in rule.arguments:
            options[arg] = "[{0}]".format(arg)

        url = url_for(rule.endpoint, **options)
        endpoints.append(urllib.parse.unquote(url))

    resp = {
        'endpoints': endpoints
    }

    return jsonify(resp)
