# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Lukas Brabec <lbrabec@redhat.com>


import flask

from flask_login import UserMixin, login_user, logout_user, current_user
from flask import request, jsonify, redirect

from oraculum import app, login_manager, oidc


class User(UserMixin):
    def __init__(self, email):
        self.id = email
        self.email = email


@login_manager.user_loader
def load_user(userid):
    return User(userid)


@app.route('/api/v1/oidc_login')
@oidc.require_login
def api_v1_oidc_login():
    email = oidc.user_getfield('email')
    user = User(email)
    login_user(user)

    token = oidc.get_access_token()

    resp = {
        'msg': 'login successful',
        'email': email
    }
    print("*"*100)
    print(flask.request.args.get('redirect'))
    print(token)
    # return redirect(flask.request.args.get('redirect'))
    return redirect(flask.request.args.get('redirect') + '/%s' % token)


@app.route('/api/v1/oidc_logout')
@oidc.require_login
def api_v1_oidc_logout():
    oidc.logout()
    logout_user()
    flask.session.clear()
    resp = {
        'msg': 'logout successful'
    }
    print("*"*100)
    print(flask.request.args.get('redirect'))
    return redirect(flask.request.args.get('redirect'))


@app.route('/api/v1/current_user')
def api_v1_get_current_user():
    resp = {
        'is_authenticated': False,
        'user': ''
    }
    if not current_user.is_authenticated:
        return jsonify(resp)

    resp['user'] = current_user.email
    resp['is_authenticated'] = True
    return jsonify(resp)


@app.route('/api/v1/secret_test')
@oidc.accept_token(require_token=True)
def api_v1_secret_test():
    resp = {
        'msg': 'wow, such secret, very hidden',
    }
    return jsonify(resp)
